/*
 * $Id: IOTester.java,v 1.2 2011/04/06 19:45:16 oboehm Exp $
 *
 * Copyright (c) 2011 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 30.03.2011 by oliver (ob@oasd.de)
 */

package patterntesting.runtime.junit;

import java.io.*;

import org.junit.Assert;

/**
 * With the IOTester you can assert if two files have equals content or
 * if two directories have the same structure with equal files.
 * <br/>
 * Note: This test is not yet tested with binary files.
 * 
 * @author oliver
 * @since 1.1 (30.03.2011)
 */
public final class IOTester {
    
    /** Utility class - no need to instantiate it. */
    private IOTester() {}
    
    /**
     * Asserts that the content of two InputStream are equal.
     * If they are not equals the first different line or byte will be shown.
     *
     * @param in1 the first InputStream
     * @param in2 the second InputStream
     * @throws AssertionError if the check fails
     */
    public static void assertContentEquals(final InputStream in1, final InputStream in2)
            throws AssertionError {
        assertContentEquals(new InputStreamReader(in1), new InputStreamReader(in2));
    }

    /**
     * Asserts that the content of two Readers are equal.
     * If they are not equals the first different line or byte will be shown.
     *
     * @param r1 the first Reader
     * @param r2 the second Reader
     * @throws AssertionError if the check fails
     */
    // TODO: shold work also for binary files
    public static void assertContentEquals(final Reader r1, final Reader r2) throws AssertionError {
        assertContentEquals(new BufferedReader(r1), new BufferedReader(r2));
    }

    /**
     * Asserts that the content of two Readers are equal.
     * If they are not equals the first different line or byte will be shown.
     *
     * @param r1 the first Reader
     * @param r2 the second Reader
     * @throws AssertionError if the check fails
     */
    public static void assertContentEquals(final BufferedReader r1, final BufferedReader r2)
            throws AssertionError {
        try {
            for (int lineNo = 1; ; lineNo++) {
                String line1 = r1.readLine();
                String line2 = r2.readLine();
                if ((line1 == null) && (line2 == null)) {
                    return;
                }
                Assert.assertEquals("line " + lineNo + " -", line1, line2);
            }
        } catch (IOException ioe) {
            throw new AssertionError(ioe);
        }
    }

}

